/*****
*
* Copyright (C) 1998 - 2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#ifndef _PLUGIN_COMMON_PRV_H
#define _PLUGIN_COMMON_PRV_H

typedef struct 
{
        struct list_head int_list;
        struct list_head ext_list;

        int already_used;
        
        plugin_generic_t *plugin;

        const char *infos;
        
        /*
         * Plugin running time and count.
         */
        double p_time;
        unsigned long p_count;

        /*
         * Plugin provided Filter rules running time & count.
         */
        double pfr_time;
        unsigned long pfr_count;

        /*
         * 
         */
        void *parent;
} plugin_container_t;

int plugin_load_from_dir(const char *dirname, int (*cb)(plugin_container_t *p));
int plugin_register(plugin_container_t *pc, struct list_head *h, const char *infos);
void plugin_run(plugin_container_t *pc, void *data);
void plugin_print_stats(plugin_container_t *pc);
void plugins_print_stats(void);
void plugins_print_opts(void);
int plugin_set_opts(char *pname, int argc, char **argv);


#define plugin_run(pc, type, arg...) do {   \
        struct timeval start, end;          \
                                            \
        gettimeofday(&start, NULL);         \
        ((type *)pc->plugin)->run(arg);     \
        gettimeofday(&end, NULL);           \
        pc->p_time += (double) end.tv_sec + (double) (end.tv_usec * 1e-6);  \
        pc->p_time -= (double) start.tv_sec + (double) (start.tv_usec * 1e-6); \
        pc->p_count++;                      \
} while (0)


#endif
