/*****
*
* Copyright (C) 1999 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

typedef struct
{
        int opt;
        int datalen;
        int len;
        int opts_len;
        int orig_len;
        int more;
        const unsigned char *snapend;
        unsigned char *opt_ptr;
        unsigned char *opt_cur;
        char err[1024];
} pktopt_t;


/*
 * Return printable data
 */
const char *prelude_GetCleanData(const unsigned char *data, size_t len);

/*
 * Packet options parsing functions :
 */

/*
 * Must be called before any other options parsing
 * related functions.
 */
void prelude_InitTcpOptParsing(pktopt_t *option, struct __tcphdr *tcp);
void prelude_InitIpOptParsing(pktopt_t *option, struct __iphdr *ip);

/*
 * return the next tcp option or -1 if there
 * is no more option to parse ( or if there is an error
 * parsing Tcp/Ip options ).
 */
int prelude_GetNextTcpOption(pktopt_t *option);
int prelude_GetNextIpOption(pktopt_t *option);

/*
 * Return the next tcp value...
 * This function can be called within a loop,
 * it will return the next value each time.
 * When there is no more value, it will return -1;
 *
 * The option should had been optained using
 * prelude_GetNextTcpOption() function which must be called
 * before.
 */
int prelude_GetNextTcpOptionValue(pktopt_t *option);
int prelude_GetNextIpOptionValue(pktopt_t *option);

/*
 * Search for the option wanted into
 * the packet contained Tcp options.
 * return -1 if not found...
 * else return 0.
 */
int prelude_SearchTcpOption(pktopt_t *option, int wanted);
int prelude_SearchIpOption(pktopt_t *option, int wanted);

