/*****
*
* Copyright (C) 1999,2000 Vandoorselaere Yoann <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "packet.h"
#include "pluginlib.h"
#include "common.h"

static char *clean_data = NULL;

void __libprelude_FreeCachedData(void);


__inline__ static void __create_CleanData(const unsigned char *data, size_t dlen) 
{
        int i, printable = 0;
        char *p;
        
        if ( clean_data != NULL )
                return;
        
        clean_data = (char *) malloc(dlen + 1);
        if ( ! clean_data ) {
                err("malloc(%d)", dlen + 1);
                return;
        }

        p = clean_data;

        for ( i = 0; i < dlen; i++ ) {
                if ( isprint(data[i]) ) {
                        printable++;
                        *p++ = data[i];
                }
        }

        clean_data = (char *) realloc(clean_data, printable + 1);
        if ( ! clean_data ) {
                err("realloc(%d)", printable - 1);
                return;
        }
        
        clean_data[printable] = '\0';
}

const char *prelude_GetCleanData(const unsigned char *data, size_t dlen) 
{
        if ( clean_data == NULL )
                __create_CleanData(data, dlen);

        return clean_data;
}

void __libprelude_FreeCachedData(void) 
{
        if ( clean_data != NULL ) {
                free(clean_data);
                clean_data = NULL;
        }
}








