/*****
*
* Copyright (C) 1998 - 2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gnome-xml/parser.h>

#include "detect.h"


plugin_detect_t *plugin_init(unsigned int id);
static void _match_run(Packet_t *packet, int depth);

typedef enum {
        cmp_sip,
        cmp_dip,
        cmp_sport,
        cmp_dport,
} cmp_kind_t;


struct _match {
        cmp_kind_t cmp_kind;
        void *data;
        const char *alert;
        struct _match *next, *child;
};


static subscribtion_t subscribtion[] = {
        { p_all, "" },
        { p_end, "" },
};


static plugin_detect_t plugin = {
        "Match",
        "Yoann Vandoorselaere",
        "... fix me ...",
        NULL,
        _match_run,
        subscribtion,
};


static unsigned int dlen;
static unsigned char *data;
static unsigned long sip;
static unsigned long dip;
static unsigned int sport;
static unsigned int dport;

static struct _match *root = NULL;

static void prepare_data(Packet_t *packet) 
{
        int i;

        dlen = 0;
        data = NULL;
        sip = dip = 0;
        sport = dport = 0;
        
        for (i = 0; packet[i].proto != p_end; i++) {

                switch (packet[i].proto) {

                case p_ip:
                        sip = packet[i].p.ip->ip_hdr.ip_src.s_addr;
                        dip = packet[i].p.ip->ip_hdr.ip_dst.s_addr;
                        break;

                case p_tcp:
                        sport = packet[i].p.tcp->tcp_hdr.th_sport;
                        dport = packet[i].p.tcp->tcp_hdr.th_dport;
                        break;

                case p_udp:
                        sport = packet[i].p.udp_hdr->uh_sport;
                        dport = packet[i].p.udp_hdr->uh_dport;
                        break;

                case p_data:
                        dlen = packet[i].p.pkt_data->size;
                        data = packet[i].p.pkt_data->data;
                        break;

                default:
                        break;

                }
        }
}



static int cmp_data(struct _match *node) 
{
        unsigned long data = (* (unsigned long *) node->data);

        /*
         * FIXME: will probably look like this in a few time :
         * return (data == ops[node->cmp_kind] ) ? 0 : -1;
         */
        
        switch (node->cmp_kind) {
                
        case cmp_sport:
                return (data == sport) ? 0 : -1;

        case cmp_dport:
                return (data == dport) ? 0 : -1;

        case cmp_sip:
                return (data == sip) ? 0 : -1;

        case cmp_dip:
                return (data == dip) ? 0 : -1;

        }

        return -1;
}



static void walk_tree(Packet_t *packet, struct _match *root) 
{
        int ret;
        
        while ( root ) {
                ret = cmp_data(root);
                if ( ret == 0 ) {
                        if ( root->alert )
                                plugin_do_report((plugin_generic_t *)&plugin,
                                                 packet,
                                                 0, "%s", root->alert);

                        walk_tree(packet, root->child);
                }
                
                root = root->next;
        }
}



static void _match_run(Packet_t *packet, int depth) 
{
        prepare_data(packet);        
        walk_tree(packet, root);
}



plugin_detect_t *plugin_init(unsigned int id) 
{
        int ret;

        parse_rule_file("/usr/local/etc/prelude/match.conf");
        
        return &plugin;
}
















