/*****
*
* Copyright (C) 1998 - 2000 Vandoorselaere Yoann <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>

#include "report.h"

#define plural(number) (number > 1) ? "s" : ""

plugin_report_t *plugin_init(unsigned int id);

static FILE *fd;



/*
 *
 */
static void log(const char *comment, ...) 
{
        va_list va;

        va_start(va, comment);
        vfprintf(fd, comment, va);
        va_end(va);
}



/*
 *
 */
static const char *switch_alert_kind(rkind_t kind) 
{
        switch(kind) {

        case maybe_faked:
                return "Could be faked";

        case maybe_not_reliable:
                return "May not be reliable";

        case normal:
                return "Should be ok";

        }

        return "";
}



/*
 *
 */
static void sysplug_run(alert_t *alert) {
        const char **ret;
        char **r = NULL;
        int i;

        log("\n\n*** %s\n", alert_date(alert));
        log("from\t\t: %s\n", alert_from(alert));
        log("description\t: %s\n", alert_desc(alert));
        log("kind\t\t: %s\n", switch_alert_kind(alert_kind(alert)));
        log("received\t: %d time%s\n", alert_count(alert), plural(alert_count(alert)));
        log("message\t\t: %s\n\n", alert_message(alert));
        
        ret = prelude_PktDump(alert_packet(alert));
        if ( ret ) 
                for ( i = 0; ret[i] != NULL; i++) 
                        log("%s\n", ret[i]);
        
        r = prelude_HexDump(alert_packet(alert));
        if ( r ) {
                log("\nData hexadecimal dump follow :\n");
                for ( i = 0; r[i] != NULL; i++ ) {
                        log("%s\n", r[i]);
                        free(r[i]);
                }
                free(r);
        }

        fflush(fd);
}


static plugin_report_t plugin = {
        "SysPlug",
        "Yoann Vandoorselaere",
        "Will syslog all attack detected.",
        NULL,
        sysplug_run,
};



plugin_report_t *plugin_init(unsigned int id) {
        time_t t;
        char *date;
        
        fd = fopen("/var/log/prelude.log", "a");
        if ( ! fd ) {
                err("error opening /var/log/prelude.log for appending");
                return NULL;
        }
        
        t = time(NULL);
        if ( t < 0 )
                err("time");

        date = ctime(&t);
        date[strlen(date) - 1] = '\0';

	return &plugin;
}












