/*****
*
* Copyright (C) 2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <syslog.h>

#include "auth.h"
#include "auth-common.h"
#include "common.h"

#define AUTH_FILE CONFIG_DIR"/prelude-report.auth"


extern char *crypt(const char *key, const char *salt);



/*
 * Replace \n by \0, as we don't want to take care of them.
 */
static char *filter_string(char *string, size_t len) 
{
        int i;
        
        for ( i = 0; i < len; i++ )
                if ( string[i] == '\n' ) string[i] = 0;

        return string;
}



/*
 * Separate user and pass from a given string.
 */
static int separate_string(const char *string, size_t slen, char **user, char **pass) 
{
        char *p = NULL;
        int len;
        
        do {
                len = strlen(string) + 1;

                if ( ! *user && (p = strstr(string, "user ")) ) 
                        *user = strdup(p + 5);
                
                if ( ! *pass && (p = strstr(string, "pass ")) ) 
                        *pass = strdup(p + 5);

                if ( ! *user && ! *pass )
                        return -1;
                
                string += len;
                
        } while ((slen -= len) != 0);

        return 0;
}




/*
 *
 */
static int cmp(const char *given_user, const char *user,
               const char *given_pass, const char *pass) 
{
        int ret;

        ret = strcmp(given_user, user);
        if ( ret == 0 ) {
                if ( strcmp(pass, crypt(given_pass, SALT)) == 0) {
                        syslog(LOG_INFO, "user %s sucessfully authenticated", user);
                        return 0;
                } else {
                        syslog(LOG_INFO, "invalid password for %s", user);
                        return -1;
                }
        }

        return -1;
}



/*
 *
 */
static int check_account(const char *given_user, const char *given_pass) 
{
        FILE *fd;
        char *user, *pass;
        int line = 0, ret;
        
        fd = fopen(AUTH_FILE, "r");
        if ( ! fd ) {
                err("couldn't open %s for reading", AUTH_FILE);
                return -1;
        }

        while ( auth_read_entry(fd, &line, &user, &pass) == 0 ) {                
                ret = cmp(given_user, user, given_pass, pass);

                free(user);
                free(pass);
                
                if (ret == 0) {
                        fclose(fd);
                        return 0;
                }
        }
        
        fclose(fd);

        syslog(LOG_INFO, "user %s do not exist", given_user);

        return -1;
}



/*
 *
 */
static int get_account_infos(int sock, char **user, char **pass) 
{
        int tlen, ret;
        char buf[1024], *u = NULL, *p = NULL;

        do {
                tlen = read(sock, buf, sizeof(buf));
                if ( tlen <= 0 ) {
                        if ( tlen < 0 )
                                err("read");
                        goto err;
                }

                filter_string(buf, tlen);
                ret = separate_string(buf, tlen, &u, &p);
                if ( ret < 0 ) {
                        err("there was an error parsing the command");
                        goto err;
                }
        } while ( u == NULL || p == NULL );

        *user = u;
        *pass = p;
        
        return 0;

 err:
        if ( u ) free(u);
        if ( p ) free(p);

        return -1;
}



/*
 *
 */
int __auth_check(int sock) 
{
        int ret;
        char *user = NULL, *pass = NULL;
        
        ret = get_account_infos(sock, &user, &pass);
        if ( ret < 0 ) {
                err("couldn't get account infos (username || / && password)");
                return -1;
        }
        
        ret = check_account(user, pass);
        if ( ret < 0 ) 
                err("login infos not valid");

        free(user);
        free(pass);
        
        return ret;
}



/*
 *
 */
int __auth_new_account(void) 
{
        return _auth_create_account(AUTH_FILE, 1);
}



/*
 *
 */
int __auth_init(void) 
{
        return auth_file_exist_or_create(AUTH_FILE, 1);
}





