/*****
*
* Copyright (C) 1998 - 2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <unistd.h>
#include <rpc/xdr.h>
#include <syslog.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "packet.h"
#include "alert.h"
#include "alert-prv.h"
#include "rxdr.h"
#include "common.h"
#include "print.h"
#include "auth.h"
#include "cnx.h"
#include "plugin-common.h"
#include "plugin-report.h"

static XDR xdrs;
static char mem[8192];
extern struct list_head __report_plugins;
static Packet_t packet[MAX_PKTDEPTH];

extern void __FreeCachedData(void);
extern void __libprelude_FreeCachedData(void);



/*
 *
 */
static int decode_alert(alert_t *alert) 
{
        int ret;

        memset(alert, 0, sizeof(alert_t));
        memset(packet, 0, sizeof(packet));
        
        alert_packet(alert) = packet;

        ret = __rxdr_decode(&xdrs, alert);
        if ( ret < 0 ) {
                p_print(stderr, "There was an error decoding the DetectReport structure.\n");
                return -1;
        }
                
        return 0;
}



/*
 *
 */
static void free_alert(alert_t *alert) 
{
        if ( alert_from(alert) )
                free(alert_from(alert));
                
        if ( alert_desc(alert) )
                free(alert_desc(alert));

        if ( alert_date(alert) ) 
                free(alert_date(alert));
}



#include <assert.h>
/*
 *
 */
static int wait_report(int sock) 
{
        int ret, len;
        alert_t alert;

        while ( 1 ) {
                if ( ! xdrrec_skiprecord(&xdrs) ) {
                        err("There was an error waiting / reading next report");
                        return -1;
                }
                                                
                ret = decode_alert(&alert);
                if ( ret < 0 ) {
                        p_print(stderr, "There was an error decoding the XDR alert.\n");
                        free_alert(&alert);
                        continue;
                }

                report_plugins_run(&alert);
                
                __FreeCachedData();
                __libprelude_FreeCachedData();
                
                free_alert(&alert);

        }

        xdr_destroy(&xdrs);  

        return 0;
}


static int do_read(char *handle, char *buf, int len) 
{
        int ret;
        int sock = *((int *) handle);

        ret = read(sock, buf, len);
        printf("read returned %d\n", ret);
        
        return (ret == 0) ? -1 : ret;
}


/*
 *
 */
int __cnx_handle(int sock) 
{
        int ret = 0;
        alert_t alert;

        ret = __auth_check(sock);
        if ( ret < 0 ) {
                write(sock, "failed", 7);
                return -1;
        }

        write(sock, "ok", 2);

        xdrs.x_op = XDR_DECODE;
        xdrrec_create(&xdrs, 0, 0, (char *)&sock, do_read, NULL);
        
        return wait_report(sock);
}







