/*****
*
* Copyright (C) 1999 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "config-engine.h"
#include "pconfig.h"
#include "auth.h"


#define CONF CONFIG_DIR"prelude.conf"

struct report_config config;
int config_quiet;

int __config_init(int argc, char **argv);

static void configure_listen_address(config_t *cfg) 
{
        const char *ret;
        
        if ( config.addr )
                return;

        ret = config_get(cfg, "Prelude Report", "listen");
        if ( ! ret ) {
                fprintf(stderr, "No report server specified in config file or command line.\n");
                exit(1);
        }

        config.addr = ret;
}

static void configure_listen_port(config_t *cfg) 
{
        const char *ret;
        
        if ( config.port != 0 )
                return;
        
        ret = config_get(cfg, "Prelude Report", "port");
        if ( ! ret ) {
                fprintf(stderr, "No report server port specified in config gile or command line.\n");
                exit(1);
        }
        
        config.port = atoi(ret);
}

static void configure_as_daemon(config_t *cfg) 
{
        const  char *ret;
        
        ret = config_get(cfg, "Prelude Report", "daemon");
        if ( ret ) {
                if ( strcmp(ret, "true") == 0 )
                        config.daemonize = 1;
        }
}

static void configure_quiet(config_t *cfg) 
{
        const char *ret;
        
        ret = config_get(cfg, "Prelude Report", "quiet");
        if ( ret ) {
                if ( strcmp(ret, "true") == 0 )
                        config_quiet = 1;
        }

}


int __config_init(int argc, char **argv) {
	int c;
        config_t *cfg;

	/* Default */
	config.addr = NULL;
	config.port = 0;
	config_quiet = 0;
	config.daemonize = 0;

	while ( (c = getopt(argc, argv, "l:p:uqdh")) != -1 ) {
		switch (c) {
                case 'u':
                        exit(__auth_new_account());
                        
                case 'l':
                        config.addr = optarg;
                        break;
                case 'p':
                        config.port = atoi(optarg);
                        break;
                case 'q':
                        config_quiet = 1;
                        break;
                case 'd':
                        config.daemonize = 1;
                        break;
                case 'h':
                        fprintf(stderr, "Options :\n");
                        fprintf(stderr, "\t-l listen address.\n");
                        fprintf(stderr, "\t-p listen port.\n");
                        fprintf(stderr, "\t-q quiet mode.\n");
                        fprintf(stderr, "\t-d daemon mode.\n");
                        fprintf(stderr, "\t-u create user.\n");
                        exit(0);
		}
	}

        cfg = config_open(CONF);
        if ( ! cfg ) {
                err("couldn't open config file %s", CONF);
                return -1;
        }
        
        configure_listen_address(cfg);
        if ( strcmp(config.addr, "unix") != 0 ) 
                configure_listen_port(cfg);
        
        configure_as_daemon(cfg);
        configure_quiet(cfg);
        
        return 0;
}



