/*****
*
* Copyright (C) 1998,1999,2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <syslog.h>
#include <signal.h>

#include "pconfig.h"
#include "print.h"
#include "auth.h"
#include "server.h"
#include "common.h"
#include "packet.h"
#include "alert.h"
#include "alert-prv.h"
#include "plugin-common.h"
#include "plugin-report.h"

static void cleanup(int sig) 
{
        fprintf(stderr, "Caught signal %d.\n", sig);
        
        /*
         * If we are using unix socket,
         * we delete the socket.
         */
        unlink(UNIX_SOCK);

        /*
         * Now we reset the signal
         * we caught to it's default behavior
         */
        signal(sig, SIG_DFL);

        /*
         * We resend the signal we just caught,
         * this time, it is directly handled
         * by the kernel.
         */
        raise(sig);
}

int main(int argc, char **argv)
{
        do_init(__config_init(argc, argv),
                "Reading Prelude Report configuration");


        do_init(__auth_init(),
                "Initializing authentication module");

        do_init(report_plugins_init(REPORT_PLUGIN_DIR),
                "Initializing report plugins");
        
        openlog("prelude-report", LOG_NDELAY, LOG_DAEMON);

        signal(SIGTERM, cleanup);
        signal(SIGINT, cleanup);
        signal(SIGSEGV, cleanup);

        do_init(__server_start(),
                "Starting report server");
                
	exit(0);	
}







