/*****
*
* Copyright (C) 1999,2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <netinet/in.h>
#include <sys/un.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <syslog.h>
#include <signal.h>

#include "pconfig.h"
#include "common.h"
#include "print.h"
#include "config.h"
#include "server.h"
#include "cnx.h"



extern struct report_config config;


#ifdef HAVE_TCPD_H

  #include <tcpd.h>
  int allow_severity = LOG_INFO, deny_severity = LOG_NOTICE;

#endif



#ifdef HAVE_TCPD_H

/*
 *
 */
static int tcpd_auth(int clnt_sock) 
{
        struct request_info request;
        int ret;
        
        request_init(&request, RQ_DAEMON, "prelude-report", RQ_FILE, clnt_sock, 0);
        
        fromhost(&request);
        ret = hosts_access(&request);
        if ( ! ret ) {
                syslog(deny_severity,
                       "prelude-report: refused connect from %s", eval_client(&request));
                close(clnt_sock);
                return -1;
        }

        syslog(allow_severity, "prelude-report: connect from %s", eval_client(&request));
        
        return 0;
}
#endif



/*
 *
 */
static const char *get_sock_addr(int sock) 
{
        int ret;
        struct sockaddr_in addr;
        int len = sizeof(addr);
        
        ret = getpeername(sock, &addr, &len);
        if ( ret < 0 ) {
                err("getpeername");
                return "";
        }

        return inet_ntoa(addr.sin_addr);
}



/*
 *
 */
static int wait_cnx(int sock, struct sockaddr *addr) 
{
        int ret, len, clnt_sock;
               
        len = sizeof(*addr);
        
        while ( 1 ) {
                clnt_sock = accept(sock, addr, &len);
                
#ifdef HAVE_TCPD_H
                ret = tcpd_auth(clnt_sock);
                if ( ret < 0 )
                        continue;
#endif
#if 0
                ret = fork();
                if ( ret < 0 ) {
                        err("fork");
                        return -1;
                }

              
                if ( ret == 0 ) {
                        close(sock);
#endif
                        syslog(LOG_INFO, "new connection from %s", get_sock_addr(clnt_sock));
                        ret = __cnx_handle(clnt_sock);
                        syslog(LOG_INFO, "%s closed connection", get_sock_addr(clnt_sock));
#if 0
                        close(clnt_sock);
                        exit(ret);
                }
#endif
                close(clnt_sock);
        }
        return 0;
}



/*
 *
 */
static void child_exit(int sig) 
{
        wait(NULL);
}



/*
 *
 */
static int generic_server(int sock, struct sockaddr *addr, size_t alen) 
{
        int ret;
        
        ret = bind(sock, addr, alen);
        if ( ret < 0 ) {
                err("bind");
		close(sock);
		return -1;
	}
        
        ret = listen(sock, 10);
        if ( ret < 0 ) {
                err("listen");
                close(sock);
                return -1;
        }

        signal(SIGCHLD, child_exit);
        
        return 0;
}



/*
 *
 */
static int unix_server_start(void) 
{
        int ret, sock;
        struct sockaddr_un addr;
        
        p_print(stdout, "\tStarting Unix report server.\n");

        sock = socket(AF_UNIX, SOCK_STREAM, 0);
        if ( sock < 0 ) {
		fprintf(stderr, "socket: %s.\n", strerror(errno));
		return -1;
	}
        
        addr.sun_family = AF_UNIX;
        strncpy(addr.sun_path, UNIX_SOCK, sizeof(addr.sun_path));

        ret = generic_server(sock, (struct sockaddr *) &addr, sizeof(addr));
        if ( ret < 0 ) 
                return -1;

        return wait_cnx(sock, (struct sockaddr *)&addr);
}



/*
 *
 */
static int inet_server_start(void) 
{
        int ret, sock;
        struct sockaddr_in addr;

        p_print(stdout, "\tStarting Tcp report server.\n" );

        sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
        if ( sock < 0 ) {
                err("socket");
                return -1;
        }
        
        addr.sin_family = AF_INET;
        addr.sin_port = htons(config.port);
        addr.sin_addr.s_addr = inet_addr(config.addr);

        ret = generic_server(sock, (struct sockaddr *) &addr, sizeof(addr));
        if ( ret < 0 ) 
                return -1;
        
        return wait_cnx(sock, (struct sockaddr *)&addr);
}


/*
 *
 */
int __server_start(void) {
	int ret;
        
        ret = strcmp(config.addr, "unix");
        if ( ret == 0 )
                ret = unix_server_start();
        else
                ret = inet_server_start();
               
        return ret;
}











