/*****
*
* Copyright (C) 2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "auth.h"
#include "auth-common.h"
#include "common.h"


#define AUTH_FILE CONFIG_DIR"/prelude.auth"


/*
 * Send authentication informations (user, pass) on socket.
 */
static int write_auth_infos(int sock, const char *user, const char *pass) 
{
        char buf[1024];
        int ret;
        
        snprintf(buf, sizeof(buf), "user %s\n", user);
        ret = write(sock, buf, strlen(buf));
        if ( ret < 0 ) {
                err("error while writing authentication infos on socket");
                return -1;
        }
                
        snprintf(buf, sizeof(buf), "pass %s\n", pass);
        ret = write(sock, buf, strlen(buf));
        if ( ret < 0 ) {
                err("error while writing authentication infos on socket");
                return -1;
        }

        return 0;
}



/*
 * Read authentication result (suceed or failed) on socket.
 */
static int read_auth_result(int sock) 
{
        char buf[1024];
        int ret;
        
        ret = read(sock, buf, sizeof(buf));
        if ( ret < 0 ) {
                err("error while reading authentication result on socket");
                return -1;
        }

        return strcmp(buf, "ok");
}



/*
 * Do the authentication procedure :
 * sending authentication informations and reading the result.
 */
static int do_auth(int sock, const char *user, const char *pass) 
{
        int ret;

        ret = write_auth_infos(sock, user, pass);
        if ( ret < 0 ) 
                return -1;

        ret = read_auth_result(sock);
        if ( ret < 0 ) {
                fprintf(stderr, "Authentication to report server failed.\n");
                return -1;
        }

        return 0;
}



/*
 * Public function used in order to start the authentication
 * process (client side), open the authentication file, read
 * an entry and try to authenticate.
 */
int auth_client(int sock) 
{
        FILE *fd;
        int ret, line;
        char *user, *pass;

        fd = fopen(AUTH_FILE, "r");
        if (! fd ) {
                err("couldn't open authentication file %s", AUTH_FILE);
                return -1;
        }

        ret = auth_read_entry(fd, &line, &user, &pass);
        if ( ret < 0 )
                err("couldn't read authentication file %s", AUTH_FILE);
        else 
                ret = do_auth(sock, user, pass);
        
        fclose(fd);
        if (user) free(user);
        if (pass) free(pass);

        return ret;
}



/*
 * Create a new account under authentication file 'AUTH_FILE',
 * and not crypted (we are the client).
 */
int auth_create_account(void) 
{
        return _auth_create_account(AUTH_FILE, 0);
}



/*
 * Initialize the authentication module,
 */
int auth_init(void) 
{
        return auth_file_exist_or_create(AUTH_FILE, 0);
}



        
        

        

        
