/*****
*
* Copyright (C) 1998, 1999, 2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <stdlib.h>
#include <signal.h>

#include "pconfig.h"
#include "print.h"
#include "daemonize.h"
#include "capture.h"
#include "hostdb.h"
#include "packet.h"
#include "auth.h"
#include "common.h"
#include "list.h"
#include "plugin-common.h"
#include "plugin-common-prv.h"
#include "detect-plugins.h"
#include "alert.h"
#include "alert-prv.h"
#include "rsend.h"

Pconfig_t config;

static void sighandler(int sig)
{
	fprintf(stderr, "\n\nCaught signal %d.\n", sig);

        signal(sig, SIG_DFL);
        
        capture_stats();
	capture_stop();
        plugins_print_stats();
        
	putc('\n', stderr);

        /*
         * Currently, exit with status 2 for gprof analysis.
         */
#if 0
        raise(sig);
#endif    
        exit(2);
}

int main(int argc, char **argv)
{

        if (getuid() != 0 || geteuid() != 0) {
                fprintf(stderr, "\n\nPrelude need root access in order to run.\n\n");
		exit(1);
	}

        p_print(stderr, "\nPrelude, (c) 1998 - 2000 Vandoorselaere Yoann.\n");
        p_print(stderr, "Developed under the GPL license.\n\n");
        
        /*
         * Detections plugins initialization
         */
        do_init(detect_plugins_init(DETECT_PLUGIN_DIR),
                "Initializing detections plugins.");

        
        do_init(pconfig_set(argc, argv), "Reading configuration");
        
        
        do_init(auth_init(),
                "Initializing authentication module");
        
        /*
         * Prelude connection hash tables initialization.
         */
        do_init(hostdb_init(),
                "Initializing hosts connections hash table.");

        /*
         * Connect to the report server, for future report sending.
         */
        do_init(rsend_init(config.addr, config.port),
                "Initializing connection to report server.");
        
        /*
         * Creation of signal handler.
         */
	signal(SIGTERM, sighandler);
	signal(SIGINT, sighandler);
	signal(SIGSEGV, sighandler);

        /*
         * Start prelude as a daemon if asked.
         */
        if ( config.daemonize == 1 )
                do_init(daemonize(),
                        "Starting Prelude as a daemon.");
        
        /*
         * Prelude capture begin here.
         */
        do_init(capture_start(),
                "Initializing packet capture");
        
        capture_stats();
        capture_stop();
        
	exit(0);
}










