/*****
*
* Copyright (C) 2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <unistd.h>

#include "common.h"
#include "rsave.h"

#define RFILE "/var/spool/prelude/report"


static FILE *fd = NULL;
static int inside = 0;

/*
 * Close the backup fd and remove backup file.
 */
static void destroy_backup(void) 
{
        fclose(fd);
        fd = NULL;
        unlink(RFILE);
}



/*
 * Create the backup fd if it do not exist,
 * else, set it up to the end of the stream.
 */
static int setup_fd(void) 
{
        int ret;
        
        if ( ! fd ) {
                fd = fopen(RFILE, "a+");
                if ( ! fd ) {
                        err("opening %s for appending", RFILE);
                        return -1;
                }         
        } else {
                ret = fseek(fd, 0, SEEK_END);
                if ( ret < 0 ) {
                        err("couldn't set stream to end of file");
                        return -1;
                }
        }
        
        return 0;
}



/*
 * Rewind stream specified by fd of number of byte 'byte'.
 */
static void advance_stream(FILE *fd, long byte) 
{
        int ret;
        long offset;

        offset = ftell(fd) + byte;
        
        ret = fseek(fd, offset, SEEK_SET);
        if ( ret < 0 ) 
                err("couldn't advance %s of %ld byte%s.\n",
                    RFILE, byte, (byte > 1) ? "s" : "");
}



/*
 * Save a report into the backup fd,
 * if there is an error writing the second part of the report,
 * rewind the file stream to the offset it was before we started
 * writing this report.
 */
static int save_report(const char *buf, const int len) 
{
        int ret;
        
        ret = fwrite(buf, sizeof(char), len, fd);
        if ( ret < 0 ) {
                err("fwrite");
                return -1;
        }
        
        return 0;
}



/*
 * backup a report.
 */
int backup_report(const char *buf, int len) 
{
        int ret;

        if ( inside == 1 )
                return 0;
        
        ret = setup_fd();
        if ( ret < 0 )
                return -1;

        return save_report(buf, len);
}



/*
 * return the number of character read,
 * 0 for end of file, -1 for error.
 */
int backout_report(void *handle, int (*cb)(void *handle, char *buf, int len)) 
{
        int ret, r;
        static long tot = 0;
        char buf[8192];

        if ( tot != 0 )
                advance_stream(fd, tot);
        else
                rewind(fd);
        
        while ( (ret = fread(buf, sizeof(char), sizeof(buf), fd)) ) {
                inside = 1;

                r = cb(handle, buf, ret);
                if ( r < 0 )
                        return -1;

                inside = 0;
                tot += ret;
        }

        if ( ret < 0 )
                err("there was an error reading backup");
        
        destroy_backup();
        tot = 0;
        
        return 0;
}








